<template>
  <Popup
    ref="pop"
    :title="title"
    left="calc(100% - 420px)"
    @close="close(true)"
    width="400px"
    showBtn="true"
    @yes="addData"
    @cancel="close(false)"
  >
    <el-form ref="form" :model="data" label-width="105px">
      <el-form-item label="服务地址:">
        <el-input v-model="data.urls" :title="data.urls" disabled></el-input>
      </el-form-item>
      <!-- <el-form-item label="token:" v-if="data.sourceType === 'tdmap'">
        <el-input v-model="tokne" :title="tokne"></el-input>
      </el-form-item> -->
      <div class="mapList scrollbar">
        <el-collapse :value="activeNames">
          <el-collapse-item
            v-for="(mapCollection, index) in mapList"
            :title="mapCollection.name"
            :name="mapCollection.name"
            :key="index"
          >
            <div
              class="map"
              v-for="(map, mapIndex) in mapCollection.list"
              :key="mapIndex"
              @click="addImageLayer(map, mapCollection)"
            >
              <el-image
                style="width: 60px; height: 60px"
                :src="require(`@assets/imageLayer/${map.image}`)"
                fit="fill"
              ></el-image>
              <span>{{ map.name }}</span>
            </div>
          </el-collapse-item>
        </el-collapse>
      </div>
    </el-form>
  </Popup>
</template>

<script>
import Popup from "@tools/Popup";
// 影像图层
let imagerLayer;
export default {
  name: "AddOnlineMap",
  components: {
    Popup,
  },
  mixins: [],
  data() {
    return {
      title: "在线地图",
      left: undefined,
      tokne: "",
      data: {
        id: "",
        sourceType: "",
        name: "",
        urls: "",
        layer: "",
        alpha: 1,
        maximumLevel: 26,
        zIndex: undefined,
      },
      mapCollection: undefined,
      mapList: [
        {
          name: "天地图",
          sourceType: "tdmap",
          tokne: "c53eb074c3fcba5ac86103d4d711bbe8",
          maximumLevel: 18,
          list: [
            {
              name: "天地图地图",
              image: "tdmap_map.jpg",
              urls: "http://{s}.tianditu.com/vec_w/wmts?service=wmts&request=GetTile&version=1.0.0&LAYER=vec&tileMatrixSet=w&TileMatrix={TileMatrix}&TileRow={TileRow}&TileCol={TileCol}",
            },
            {
              name: "天地图影像",
              image: "tdmap_image.jpg",
              urls: "http://{s}.tianditu.com/img_w/wmts?service=wmts&request=GetTile&version=1.0.0&LAYER=img&tileMatrixSet=w&TileMatrix={TileMatrix}&TileRow={TileRow}&TileCol={TileCol}",
            },
            {
              name: "天地图注记",
              image: "tdmap_label.jpg",
              urls: "http://{s}.tianditu.com/cva_w/wmts?service=wmts&request=GetTile&version=1.0.0&LAYER=cva&tileMatrixSet=w&TileMatrix={TileMatrix}&TileRow={TileRow}&TileCol={TileCol}",
            },
          ],
        },
        {
          name: "高德地图",
          sourceType: "gdmap",
          maximumLevel: 18,
          list: [
            {
              name: "高德地图",
              image: "gdmap_map.jpg",
              urls: "https://webst02.is.autonavi.com/appmaptile?style=7&x={x}&y={y}&z={z}",
            },
            {
              name: "高德影像",
              image: "gdmap_image.jpg",
              urls: "https://webst02.is.autonavi.com/appmaptile?style=6&x={x}&y={y}&z={z}",
            },
            {
              name: "高德注记",
              image: "gdmap_label.jpg",
              urls: "https://webst02.is.autonavi.com/appmaptile?style=8&x={x}&y={y}&z={z}",
            },
          ],
        },
        {
          name: "腾讯地图",
          sourceType: "txmap",
          maximumLevel: 18,
          list: [
            {
              name: "腾讯影像",
              image: "txmap_image.png",
              urls: "https://p2.map.gtimg.com/sateTiles/{z}/{sx}/{sy}/{x}_{reverseY}.jpg",
            },
            {
              name: "腾讯地图",
              image: "txmap_map.png",
              urls: "https://rt1.map.gtimg.com/tile?z={z}&x={x}&y={reverseY}&styleid=1",
            },
            {
              name: "腾讯地图注记",
              image: "txmap_label.png",
              urls: "https://rt1.map.gtimg.com/tile?z={z}&x={x}&y={reverseY}&styleid=2",
            },
            {
              name: "腾讯地图透明",
              image: "txmap_tm.png",
              urls: "https://rt1.map.gtimg.com/tile?z={z}&x={x}&y={reverseY}&styleid=3",
            },
            {
              name: "腾讯地图深蓝",
              image: "txmap_dark.png",
              urls: "https://rt1.map.gtimg.com/tile?z={z}&x={x}&y={reverseY}&styleid=4",
            },
          ],
        },
        {
          name: "百度地图",
          sourceType: "bdmap",
          urls: "http://api{s}.map.bdimg.com/customimage/tile?&x={x}&y={y}&z={z}&scale=1&customid={style}",
          list: [
            {
              name: "百度地图",
              image: "baidu_map.jpg",
              style: "normal",
            },
            {
              name: "百度清新蓝",
              image: "baidu_qxl.jpg",
              style: "light",
            },
            {
              name: "百度精简",
              image: "baidu_jj.jpg",
              style: "googlelite",
            },
            {
              name: "百度自然绿",
              image: "baidu_zrl.jpg",
              style: "grassgreen",
            },
            {
              name: "百度红色警戒",
              image: "baidu_red.jpg",
              style: "redalert",
            },
            {
              name: "百度浪漫粉",
              image: "baidu_pink.jpg",
              style: "pink",
            },
            {
              name: "百度青春绿",
              image: "baidu_qcl.jpg",
              style: "darkgreen",
            },
            {
              name: "百度午夜蓝",
              image: "baidu_wyl.jpg",
              style: "midnight",
            },
            {
              name: "百度高端灰",
              image: "baidu_gdh.jpg",
              style: "grayscale",
            },
            {
              name: "百度黑夜",
              image: "baidu_dark.jpg",
              style: "dark",
            },
            {
              name: "百度强边界",
              image: "baidu_qbj.jpg",
              style: "hardedge",
            },
          ],
        },
        {
          name: "ArcGIS",
          sourceType: "arcgis",
          list: [
            {
              name: "ArcGIS影像",
              image: "arcgis_image.png",
              urls: "https://services.arcgisonline.com/ArcGIS/rest/services/World_Imagery/MapServer",
            },
            {
              name: "ArcGIS街道地图",
              image: "arcgis_map.png",
              urls: "https://services.arcgisonline.com/ArcGIS/rest/services/World_Street_Map/MapServer",
            },
          ],
        },
        {
          name: "GeoQ",
          sourceType: "arcgis",
          GCJ02: true,
          list: [
            {
              name: "GeoQ水系",
              image: "geoq_sx.png",
              urls: "http://thematic.geoq.cn/arcgis/rest/services/ThematicMaps/WorldHydroMap/MapServer",
            },
            {
              name: "GeoQ行政区划",
              image: "geoq_xzqh.png",
              urls: "http://thematic.geoq.cn/arcgis/rest/services/ThematicMaps/administrative_division_boundaryandlabel/MapServer",
            },
            {
              name: "GeoQ彩色",
              image: "geoq_cs.png",
              urls: "http://map.geoq.cn/ArcGIS/rest/services/ChinaOnlineCommunity/MapServer",
            },
            {
              name: "GeoQ暖色",
              image: "geoq_ns.png",
              urls: "http://map.geoq.cn/ArcGIS/rest/services/ChinaOnlineStreetWarm/MapServer",
            },
            {
              name: "GeoQ灰色",
              image: "geoq_hs.png",
              urls: "http://map.geoq.cn/arcgis/rest/services/ChinaOnlineStreetGray/MapServer",
            },
            {
              name: "GeoQ蓝黑",
              image: "geoq_dark.png",
              urls: "http://map.geoq.cn/arcgis/rest/services/ChinaOnlineStreetPurplishBlue/MapServer",
            },
            {
              name: "GeoQ英文",
              image: "geoq_yw.png",
              urls: "http://map.geoq.cn/arcgis/rest/services/ChinaOnlineCommunityENG/MapServer",
            },
          ],
        },
      ],
    };
  },
  computed: {
    activeNames() {
      return this.mapList.map((item) => {
        return item.name;
      });
    },
  },
  methods: {
    // 关闭弹窗
    close(isCloseBtn, removeLayer = true) {
      removeLayer && this.removeImageLayer();

      // 重置data值
      Object.assign(this.$data, this.$options.data());
      !isCloseBtn && this.$refs.pop.close();
    },
    // 打开弹窗
    open() {
      this.close(true);
      this.$refs.pop.open();
    },
    changeToken(token) {
      this.mapCollection.tokne = token;
    },
    removeImageLayer() {
      imagerLayer && imagerLayer.deleteObject();
      imagerLayer = undefined;
    },
    addImageLayer(map, mapCollection) {
      if (
        this.data.name === map.name &&
        (this.data.urls === map.urls || this.data.urls === mapCollection.urls)
      ) {
        return;
      }
      this.removeImageLayer();
      if (!this.data.id) {
        this.data.id = sgworld.Core.getuid();
      }
      this.data.name = map.name;
      this.data.sourceType = mapCollection.sourceType;
      this.data.urls = map.urls || mapCollection.urls;
      this.data.maximumLevel =
        map.maximumLevel || mapCollection.maximumLevel || 26;

      this.mapCollection = mapCollection;
      if (mapCollection.sourceType === "tdmap") {
        this.tokne = mapCollection.tokne;
        //添加天地图影像
        imagerLayer = sgworld.Creator.createWebMapTileServerImageLayer(
          "天地图",
          {
            id: this.data.id,
            url: map.urls + "&tk=" + this.tokne,
            maximumLevel: this.data.maximumLevel,
            subdomains: ["t0", "t1", "t2", "t3", "t4", "t5", "t6", "t7"],
          },
          "0",
          undefined,
          true
        );
      } else if (mapCollection.sourceType === "arcgis") {
        //添加arcgis影像
        imagerLayer = sgworld.Creator.createArcGisImageryLayer(
          "ARCGIS",
          {
            id: this.data.id,
            url: map.urls,
            enablePickFeatures: false,
            usePreCachedTilesIfAvailable: mapCollection.GCJ02 ? false : true, // 是否使用服务的内置的切片方案，偏移纠正需要使用如下自定义的方案
            tilingScheme: mapCollection.GCJ02
              ? sgworld.Core.getOffsetTilingScheme()
              : undefined, // 偏移纠正
          },
          "0",
          undefined,
          true
        );
      } else if (mapCollection.sourceType === "gdmap") {
        imagerLayer = sgworld.Creator.createUrlTemplateImageryProvider(
          "高德",
          {
            id: this.data.id,
            url: map.urls,
            maximumLevel: this.data.maximumLevel,
            tilingScheme: sgworld.Core.getOffsetTilingScheme(), // 偏移纠正
          },
          "0",
          undefined,
          true,
          ""
        );
      } else if (mapCollection.sourceType === "txmap") {
        //腾讯地图添加
        imagerLayer = sgworld.Creator.createUrlTemplateImageryProvider(
          "腾讯地图",
          {
            id: this.data.id,
            url: map.urls,
            maximumLevel: this.data.maximumLevel,
            customTags: {
              sx: function (imageryProvider, x, y, level) {
                return x >> 4;
              },
              sy: function (imageryProvider, x, y, level) {
                return ((1 << level) - y) >> 4;
              },
            },
            tilingScheme: sgworld.Core.getOffsetTilingScheme(), // 偏移纠正
          },
          "0",
          undefined,
          true
        );
      } else if (mapCollection.sourceType === "bdmap") {
        mapCollection.style = map.style;
        //百度地图添加
        imagerLayer = sgworld.Creator.createImageryProvider(
          "百度",
          "BaiduMap",
          {
            id: this.data.id,
            style: map.style,
          },
          "0",
          undefined,
          true
        );
      }
    },
    // 添加数据
    addData() {
      if (this.data.urls && this.data.sourceType) {
        let data = {
          id: window.sgworld.Core.getuid(),
          ...this.data,
          Level: [0, 26],
        };
        if (data.sourceType === "tdmap") {
          data.urls += "&tk=" + this.tokne;
        } else if (data.sourceType === "bdmap") {
          data.style = this.mapCollection.style;
        }
        if (this.mapCollection.GCJ02) {
          data.GCJ02 = true;
        }

        data.item = imagerLayer;
        imagerLayer = undefined;

        this.$emit("success", data);
        this.close(false, false);
      } else {
        this.$message("请选择地图");
      }
    },
  },
};
</script>

<style scoped lang="less">
.el-form {
  width: 400px;
  margin-top: 20px;
  margin-right: 10px;

  /deep/ .el-form-item__label {
    color: #fff;
    font-size: 18px;
  }

  /deep/ .el-select {
    width: 100%;
  }

  .el-collapse {
    border: unset !important;

    /deep/ .el-collapse-item__header {
      background: transparent;
      color: #fff;
      padding-left: 16px;
      height: 30px;
      border-bottom: unset;
    }

    /deep/ .el-collapse-item__wrap {
      background-color: transparent;
      border-bottom: 1px solid rgba(32, 160, 255, 0.3);

      .el-collapse-item__content {
        padding-bottom: 0;
        color: #fff;
      }
    }
  }

  .mapList {
    border-top: 1px solid rgba(32, 160, 255, 0.3);
    height: 200px;
    .map {
      display: inline-block;
      cursor: pointer;
      margin: 5px;
      text-align: center;
      span {
        display: block;
        font-size: 12px;
      }
      .el-image {
        border: 2px solid #fff;
        border-radius: 4px;
      }
    }
  }
}
</style>
